# -*- coding: utf-8 -*-
"""
pcapy clone in pure python
This module aims to support exactly the same interface as pcapy,
so that it can be used as a drop-in replacement. This means the module
does not and probably will never support live captures. Offline file
support should match the original though.
"""

# Copyright 2010 Stanisław Pitucha. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, are
# permitted provided that the following conditions are met:
# 
# 1. Redistributions of source code must retain the above copyright notice, this list of
# 	conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright notice, this list
# 	of conditions and the following disclaimer in the documentation and/or other materials
# 	provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY Stanisław Pitucha ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# The views and conclusions contained in the software and documentation are those of the
# authors and should not be interpreted as representing official policies, either expressed
# or implied, of Stanisław Pitucha.


import struct
import logging

DLT_NULL = 0
DLT_EN10MB = 1
DLT_IEEE802 = 6
DLT_ARCNET = 7
DLT_SLIP = 8
DLT_PPP = 9
DLT_FDDI = 10
DLT_ATM_RFC1483 = 11
DLT_RAW = 12
DLT_PPP_SERIAL = 50
DLT_PPP_ETHER = 51
DLT_RAW = 101
DLT_C_HDLC = 104
DLT_IEEE802_11 = 105
DLT_LOOP = 108
DLT_LINUX_SLL = 113
DLT_LTALK = 114

# For IEEE 802.15.4 DLT codes
DLT_IEEE802_15_4 = 195
DLT_IEEE802_15_4_NONASK_PHY = 215
DLT_IEEE802_15_4_NOFCS = 230


logger = logging.getLogger()

class PcapError(Exception):
    """ General Pcap module exception class """
    pass


def fixup_identical_short(short):
    """ noop for "fixing" big/little endian """
    return short


def fixup_identical_long(long_int):
    """ noop for "fixing" big/little endian """
    return long_int


def fixup_swapped_short(short):
    """ swap bytes in a 16b short """
    return ((short & 0xff) << 8) | ((short & 0xff00) >> 8)


def fixup_swapped_long(long_int):
    """ swap swapped shorts in a 32b int """
    bottom = fixup_swapped_short(long_int & 0xffff)
    top = fixup_swapped_short((long_int >> 16) & 0xffff)
    return ((bottom << 16) & 0xffff0000) | top


fixup_sets = {
    b"\xd4\xc3\xb2\xa1": (fixup_identical_short, fixup_identical_long),
    b"\xa1\xb2\xc3\xd4": (fixup_swapped_short, fixup_swapped_long),
}


def open_offline(filename):
    """ opens the pcap file indicated by `filename` and returns a Reader object """
    if filename == "-":
        import sys
        source = sys.stdin
    else:
        try:
            source = open(filename, "rb")
        except IOError as error:
            if error.args[0] == 21:
                raise PcapError("error reading dump file: %s" % (error.args[1]))
            else:
                raise PcapError("%s: %s" % (filename, error.args[1]))

    return Reader(source)


def open_live(_device, _snaplen, _promisc, _to_ms):
    raise NotImplementedError("This function is only available in pcapy")


def lookupdev():
    raise NotImplementedError("This function is only available in pcapy")


def findalldevs():
    raise NotImplementedError("This function is only available in pcapy")


def compile(_linktype, _snaplen, _filter, _optimize, _netmask):
    raise NotImplementedError("not implemented yet")


class Reader(object):
    """
    An interface for reading an open pcap file.
    This object can either read a single packet via `next()` or a series
    via `loop()` or `dispatch()`.
    """

    __GLOBAL_HEADER_LEN = 24
    __PACKET_HEADER_LEN = 16

    def __init__(self, source):
        """ creates a Reader instance from an open file object """

        self.__source = source
        header = self.__source.read(self.__GLOBAL_HEADER_LEN)
        if len(header) < self.__GLOBAL_HEADER_LEN:
            raise PcapError(
                "truncated dump file; tried to read %i file header bytes, only got %i" %
                (self.__GLOBAL_HEADER_LEN, len(header)))

        hdr_values = struct.unpack("IHHIIII", header)
        if header[:4] in fixup_sets:
            self.fixup_short, self.fixup_long = fixup_sets[header[:4]]
            logger.debug(header[:4])
            logger.debug(hdr_values)
        else:
            raise PcapError("bad dump file format")

        self.version_major, self.version_minor = [self.fixup_short(x)
                                                  for x in hdr_values[1:3]]
        self.thiszone, self.sigfigs, self.snaplen, self.network = [self.fixup_long(x)
                                                                   for x in hdr_values[3:]]

        self.last_good_position = self.__GLOBAL_HEADER_LEN
        logger.debug('pcap header: network: %s sigfigs: %s snaplen: %s network %s'%(self.thiszone, self.sigfigs, self.snaplen, self.network))

    def __loop_and_count(self, maxcant, callback):
        """
        reads up to `maxcant` packets and runs callback for each of them
        returns the number of packets processed
        """

        i = 0
        while True:
            if i >= maxcant > -1:
                break

            hdr, data = self.next()
            if hdr is None:
                break
            else:
                callback(hdr, data)

            i += 1

        return i

    def dispatch(self, maxcant, callback):
        """
        reads up to `maxcant` packets and runs callback for each of them
        returns the number of packets processed or 0 if no limit was specified
        """

        i = self.__loop_and_count(maxcant, callback)

        if maxcant > -1:
            return i
        else:
            return 0

    def loop(self, maxcant, callback):
        """
        reads up to `maxcant` packets and runs callback for each of them
        does not return a value
        """
        self.__loop_and_count(maxcant, callback)

    def next(self):
        """ reads the next packet from file and returns a (Pkthdr, data) tuple """

        header = self.__source.read(self.__PACKET_HEADER_LEN)

        if len(header) == 0:
            return (None, '')
        if len(header) < self.__PACKET_HEADER_LEN:
            raise PcapError(
                "truncated dump file; tried to read %i header bytes, only got %i" %
                (self.__PACKET_HEADER_LEN, len(header)))

        hdr_values = struct.unpack("IIII", header)
        ts_sec, ts_usec, incl_len, orig_len = [self.fixup_long(x) for x in hdr_values]

        data = self.__source.read(incl_len)
        if len(data) < incl_len:
            raise PcapError(
                "truncated dump file; tried to read %i captured bytes, only got %i" %
                (incl_len, len(data)))

        pkthdr = Pkthdr(ts_sec, ts_usec, incl_len, orig_len)
        return (pkthdr, data)

    def getnet(self):
        raise NotImplementedError("This function is only available in pcapy")

    def getmask(self):
        raise NotImplementedError("This function is only available in pcapy")

    def datalink(self):
        """ returns the datalink type used in the current file """
        return self.network

    def getnonblock(self):
        """
        shows whether the operations are nonblocking, which is always 0 for files
        """
        return 0

    def setnonblock(self, state):
        """ this has no effect on savefiles, so is not implemented in pure-pcapy """
        pass

    def dump_open(self, filename):
        """
        create a new dumper object which inherits the network and snaplen information
        from the original reader
        """
        return Dumper(filename, self.snaplen, self.network)


class Dumper(object):
    """
    Interface for pcap files, which can be used for creating new files.
    Although this is accessible only through `Reader.dump_open()` in the
    original pcapy, there's no good reason for that limitation and this object
    can be used directly. It implements some sanity checking before the packet
    is written to the file.
    Files created this way are always stored using the native byte order.
    """

    def __init__(self, filename, snaplen, network):
        """ creates a new dumper object which can be used for writing pcap files """
        self.store = open(filename, "wb")
        self.store.write(struct.pack("IHHIIII", 0xa1b2c3d4, 2, 4, 0, 0,
                                     snaplen, network))
        self.store.flush()  # have to flush, since there's no close

    def dump(self, header, data):
        """ writes a new header and packet to the file, then forces a file flush """
        if not isinstance(header, Pkthdr):
            raise PcapError("not a proper Pkthdr")

        if type(data) != bytes:
            raise PcapError("can dump only bytes")

        if header.getcaplen() != len(data):
            raise PcapError("capture length not equal to length of data")

        fields = list(header.getts()) + [header.getcaplen(), header.getlen()]
        self.store.write(struct.pack("IIII", *fields))
        self.store.write(data)
        self.store.flush()


class Pkthdr(object):
    """
    Packet header, as used in the pcap files before each data segment.
    This class is a simple data wrapper.
    """

    def __init__(self, ts_sec, ts_usec, incl_len, orig_len):
        self.ts = (ts_sec, ts_usec)
        self.incl_len = incl_len
        self.orig_len = orig_len

    def getts(self):
        """ returns a (seconds, microseconds) tuple for the capture time"""
        return self.ts

    def getcaplen(self):
        """ returns the captured length of the packet """
        return self.incl_len

    def getlen(self):
        """ returns the original length of the packet """
        return self.orig_len


class Bpf(object):
    def __init__(self):
        raise NotImplementedError("not implemented yet")

    def filter(self, packet):
        raise NotImplementedError("not implemented yet")



"""

"""
import json
import logging
from multiprocessing import Process

from ..utils.messages import Message as EventBusMessage

from amqp.exceptions import UnexpectedFrame
from kombu import Connection, Queue, Exchange, Consumer
from kombu.mixins import ConsumerMixin


class BaseConsumer(ConsumerMixin):
    DEFAULT_EXCHANGE_NAME = "amq.topic"
    AMQP_CONNECTION_TIMEOUT = 10

    def __init__(self, user, password, session, server, exchange, name, consumer_name, topics):
        """

        Args:
            user: Username
            password: User password
            session: Test session
            server: Backend for the RMQ
            exchange: RMQ exchange for sending messages
            name: Identity of the agent. Used by testing tools to identify/differentiate each agent on the session
            consumer_name: Name to easily identify a process consuming.
            topics: Topics subscriptions for the consumer
        """
        self.log = logging.getLogger(__name__)
        self.log.setLevel(logging.DEBUG)

        self.user = user
        self.password = password
        self.session = session
        self.server = server
        self.name = name
        self.consumer_name = consumer_name
        self.server_url = 'amqp://{user}:{password}@{server}/{session}'.format(user=user,
                                                                               password=password,
                                                                               session=session,
                                                                               server=server)
        self.connection = Connection(self.server_url,
                                     transport_options={'confirm_publish': True},
                                     connect_timeout=self.AMQP_CONNECTION_TIMEOUT)

        if exchange:
            self.exchange = Exchange(exchange,
                                     type="topic",
                                     durable=True)
        else:
            self.exchange = Exchange(BaseConsumer.DEFAULT_EXCHANGE_NAME,
                                     type="topic",
                                     durable=True)

        # queues created for topic subscriptions
        self.queues = []

        # handle subscriptions
        self.subscribe_to_topics(topics)

    def subscribe_to_topics(self, topic_list):
        for t in topic_list:
            queue = Queue(
                name="consumer: {name}.{consumer_name}?rkey={rkey}".format(
                    name=self.name,
                    consumer_name=self.consumer_name,
                    rkey=t
                ),
                exchange=self.exchange,
                routing_key=t,
                durable=False,
                auto_delete=True
            )

            self.queues.append(
                queue
            )

    def get_consumers(self, Consumer, channel):
        return [
            Consumer(self.queues, callbacks=[self.on_message], accept=['json']),
        ]

    def on_message(self, body, message):
        assert type(body) is dict  # assert that kombu deserialized it already
        json_body = json.dumps(body)

        self.log.debug("base on_message() callback, got: {}".format(message.delivery_info.get('routing_key')))
        msg = EventBusMessage.load(json_body, message.delivery_info.get('routing_key'))
        try:
            self._on_message(msg)
        except UnexpectedFrame as uf_err:
            self.log.error(
                uf_err.message
            )

    def _on_message(self, message):
        "Class to be overridden by children class"
        raise NotImplementedError()

    def on_consume_ready(self, connection, channel, consumers, wakeup=True, **kwargs):
        # control plane info
        for q in self.queues:
            self.log.info("Queue: {queue_name} bound to: {rkey} ".format(queue_name=q.name, rkey=q.routing_key))


class BaseController(Process):
    """

    """

    def __init__(self, name, process_args=None):
        if process_args is not None:
            super(BaseController, self).__init__(**process_args)
        else:
            super(BaseController, self).__init__()
        self.go_on = True
        self.name = name



# -*- coding: utf-8 -*-

import serial
import logging
import sys

from kombu import Exchange
from ..utils import messages

STATE_OK = 0
STATE_ESC = 1
STATE_RUBBISH = 2
SLIP_END = 'c0'
SLIP_ESC = 'db'
SLIP_ESC_END = 'dc'
SLIP_ESC_ESC = 'dd'

logging.basicConfig(level=logging.DEBUG)
log = logging.getLogger(__name__)


arrow_up = """
      _
     / \\
    /   \\
   /     \\
  /       \\
 /__     __\\   
    |   |              _ _       _      
    |   |             | (_)     | |         
    |   |  _   _ _ __ | |_ _ __ | | __        
    |   | | | | | '_ \| | | '_ \\| |/ /      
    |   | | |_| | |_) | | | | | |   <
    |   |  \__,_| .__/|_|_|_| |_|_|\_\\              
    |   |       | |           
    |   |       |_|                  
    !___!   
   \\  O  / 
    \\/|\/ 
      | 
     / \\
   _/   \\ _

"""

arrow_down = """
    ___       
   |   |       
   |   |       _                     _ _       _    
   |   |      | |                   | (_)     | |   
   |   |    __| | _____      ___ __ | |_ _ __ | | __
   |   |   / _` |/ _ \\ \\ /\\ / / '_ \\| | | '_ '\\| |/ /
   |   |  | (_| | (_) \\ V  V /| | | | | | | | |   < 
   |   |   \\__,_|\\___/ \\_/\\_/ |_| |_|_|_|_| |_|_|\_\\
   |   | 
 __!   !__,
 \\       / \O
  \\     / \/|
   \\   /    |
    \\ /    / \\
     Y   _/  _\\
"""


class SerialListener(object):
    def __init__(self, agent_name, rmq_connection, rmq_exchange="amq.topic", serial_port='/dev/ttyUSB0',
                 serial_boudrate='460800'):
        # give this thread a name
        self.name = 'SerialListener'
        self._stop = False

        # RMQ setups
        self.connection = rmq_connection
        self.producer = self.connection.Producer(serializer='json')
        self.exchange = Exchange(rmq_exchange, type="topic", durable=True)

        # serial interface Listener config
        self.dev = serial_port
        self.br = serial_boudrate
        self.agent_name = agent_name
        self.frame = ''
        self.start_frame = 0
        self.state = STATE_OK
        self.frame_slip = ''

        log.info("opening serial reader...")
        try:
            self.ser = serial.Serial(port=self.dev,
                                     baudrate=int(self.br),
                                     timeout=0.001
                                     )
        except serial.serialutil.SerialException as e:
            log.error(e)
            log.error('Does dev %s exist?' % serial_port)
            sys.exit(1)

        try:
            self.ser.flushInput()
        except Exception as e:
            log.error(e)

        self.message_read_count = 0

        # notify interface is opened
        m = messages.MsgAgentSerialStarted(
            name=self.agent_name,
            port=self.dev,
            boudrate=self.br
        )

        self.producer.publish(
            m.to_dict(),
            exchange=self.exchange,
            routing_key=messages.MsgAgentSerialStarted.routing_key.replace('*', self.agent_name)
        )

    def close(self):
        self._stop = True

    def closed(self):
        return self._stop

    def state_rubbish(self, data):
        if data.encode('hex') == SLIP_END:
            self.state = STATE_OK
            self.start_frame = 0
            self.frame = ''
            self.frame_slip = ''
        else:
            log.debug("Rubbish message dropped..")

    def state_esc(self, data):
        if data.encode('hex') != SLIP_ESC_END and data.encode('hex') != SLIP_ESC_ESC:
            self.state = STATE_RUBBISH
            self.start_frame = 0
            self.frame = ''
        else:
            self.state = STATE_OK
            if self.start_frame == 1:
                if data.encode('hex') == SLIP_ESC_ESC:
                    self.frame += SLIP_ESC
                elif data.encode('hex') == SLIP_ESC_END:
                    self.frame += SLIP_END

    def state_ok(self, data):
        if data.encode('hex') == SLIP_ESC:
            self.state = STATE_ESC
        else:
            if data.encode('hex') == SLIP_END:
                if self.start_frame == 0:
                    # start frame
                    self.start_frame = 1
                    self.frame = ''
                else:
                    # end frame
                    self.start_frame = 0
                    self.send_amqp(self.frame, self.frame_slip)
                    self.state = STATE_OK
            else:
                if self.start_frame == 1:
                    self.frame += data.encode('hex')
                else:
                    self.state = STATE_RUBBISH

    def recv_chars(self, chars):
        self.message_read_count += 1
        log.debug("Message received in serial interface. Count: %s" % self.message_read_count)
        if chars:
            for c in chars:
                if self.state == STATE_RUBBISH:
                    self.state_rubbish(c)
                    continue
                self.frame_slip += c.encode('hex')
                if self.state == STATE_ESC:
                    self.state_esc(c)
                    continue
                if self.state == STATE_OK:
                    self.state_ok(c)

    def convert_bytearray_to_intarray(self, ba):
        ia = []
        for e in ba:
            ia.append(e)
        return ia

    def send_amqp(self, data, data_slip):

        m = messages.MsgPacketSniffedRaw(
            interface_name='serial',
            data=self.convert_bytearray_to_intarray(bytearray.fromhex(data)),
            data_slip=self.convert_bytearray_to_intarray(bytearray.fromhex(data_slip)),
        )

        self.frame_slip = ''

        self.producer.publish(
            m.to_dict(),
            exchange=self.exchange,
            routing_key="fromAgent.{agent_name}.802154.serial.packet.raw".format(agent_name=self.agent_name)
        )
        print(arrow_up)
        log.info('\n # # # # # # # # # # # # SERIAL INTERFACE # # # # # # # # # # # # ' +
                 '\n data packet Serial -> EventBus' +
                 '\n' + m.to_json() +
                 '\n # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # '
                 )

    def run(self):
        log.info("starting serial reader thread...")
        try:
            while not self.closed():
                numbytes = self.ser.inWaiting()
                if numbytes > 0:
                    output = self.ser.read(numbytes)  # read output
                    self.recv_chars(output)
        except Exception as e:
            log.error('Error found while processing received data stream in serial: %s' % str(e))
            sys.exit(1)



# -*- coding: utf-8 -*-

"""

"""
import json
import logging
import sys
import datetime
from kombu import Producer

from .base import BaseController, BaseConsumer
from ..utils.opentun import OpenTunLinux, OpenTunMACOS
from ..utils import arrow_up, arrow_down
from ..utils import messages


class TunConsumer(BaseConsumer):
    """
    Tun interface consumer:
        - creates tunnel interface (RAW_IP)
        - injects IPv6 packets coming from event bus into tun interface
        - captures and forwards packets from tun interface to event bus
    """

    def __init__(self, user, password, session, server, exchange, name, consumer_name,
                 force_bootstrap, ipv6_host, ipv6_prefix,
                 re_route_packets_prefix, re_route_packets_host, re_route_packets_if):

        self.dispatcher = {
            messages.MsgAgentTunStart: self.handle_start,
            messages.MsgPacketInjectRaw: self.handle_raw_packet_to_inject,
        }
        self.tun = None
        self.packet_count = 0

        subscriptions = [
            messages.MsgAgentTunStart.routing_key.replace('*', name),  # default rkey is "toAgent.*.ip.tun.start"
            messages.MsgPacketInjectRaw.routing_key.replace('*', name)
        ]

        super(TunConsumer, self).__init__(user, password, session, server, exchange, name, consumer_name, subscriptions)

        if force_bootstrap:
            self.handle_start(
                messages.MsgAgentTunStart(
                    name=name,
                    ipv6_host=ipv6_host,
                    ipv6_prefix=ipv6_prefix,
                    ipv6_no_forwarding=False if re_route_packets_prefix and re_route_packets_host else True,
                    ipv4_host=None,
                    ipv4_network=None,
                    ipv4_netmask=None,
                    re_route_packets_if=re_route_packets_if,
                    re_route_packets_prefix=re_route_packets_prefix,
                    re_route_packets_host=re_route_packets_host
                )
            )

    def _on_message(self, message):
        msg_type = type(message)
        assert msg_type in self.dispatcher.keys(), 'Event message couldnt be dispatched %s' % repr(message)
        self.log.debug(
            "Consumer specialized handler <{consumer_name}> got: {message}".format(
                consumer_name=self.consumer_name,
                message=repr(message)
            )
        )
        self.dispatcher[msg_type](message)

    def _publish_agent_tun_started_message(self):
        assert self.tun is not None

        def errback(exc, interval):
            self.log.error('Error: %r', exc, exc_info=1)
            self.log.info('Retry in %s seconds.', interval)

        # get config from tun
        conf_params = self.tun.get_tun_configuration()
        conf_params.update({'name': self.name})

        # publish message in event bus
        msg = messages.MsgAgentTunStarted(**conf_params)
        self.log.debug('Publishing %s' % repr(msg))

        producer = Producer(self.connection, serializer='json')
        publish = self.connection.ensure(producer, producer.publish, errback=errback, max_retries=3)

        publish(
            body=msg.to_dict(),
            exchange=self.exchange,
            routing_key='fromAgent.{0}.ip.tun.started'.format(self.name)
        )

    def handle_start(self, msg):
        """
        Function that will handle tun start event emitted coming from backend
        """
        if self.tun is not None:
            self.log.warning('Received open tun control message, but TUN already created')

        else:
            self.log.info('starting tun interface')
            try:
                ipv6_host = msg.ipv6_host
                ipv6_prefix = msg.ipv6_prefix
                ipv6_no_forwarding = msg.ipv6_no_forwarding
                ipv4_host = msg.ipv4_host
                ipv4_network = msg.ipv4_network
                ipv4_netmask = msg.ipv4_netmask
                re_route_packets_if = msg.re_route_packets_if
                re_route_packets_prefix = msg.re_route_packets_prefix
                re_route_packets_host = msg.re_route_packets_host

            except AttributeError as ae:
                self.log.error(
                    'Wrong message format: {0}'.format(repr(msg))
                )
                return

            params = {
                'rmq_connection': self.connection,
                'rmq_exchange': self.exchange,
                'name': self.name,
                'ipv6_host': ipv6_host,
                'ipv6_prefix': ipv6_prefix,
                'ipv4_host': ipv4_host,
                'ipv4_network': ipv4_network,
                'ipv4_netmask': ipv4_netmask,
                'ipv6_no_forwarding': ipv6_no_forwarding,
                're_route_packets_if': re_route_packets_if,
                're_route_packets_prefix': re_route_packets_prefix,
                're_route_packets_host': re_route_packets_host
            }

            if sys.platform.startswith('win32'):
                self.log.error('Agent TunTap not yet supported for windows')
                sys.exit(1)

            elif sys.platform.startswith('linux'):
                self.log.info('Starting open tun [linux]')
                self.tun = OpenTunLinux(**params)

            elif sys.platform.startswith('darwin'):
                self.log.info('Starting open tun [darwin]')
                self.tun = OpenTunMACOS(**params)
            else:
                self.log.error('Agent TunTap not yet supported for: {0}'.format(sys.platform))
                sys.exit(1)

        self._publish_agent_tun_started_message()

    def handle_raw_packet_to_inject(self, message):
        """
        Handles data messages to be injected in network interface
        """
        if self.tun is None:
            self.log.error("Cannot handle data packet, no tun interface yet configured")
            return

        self.packet_count += 1

        self.log.info("Message received from testing tool. Injecting in Tun. Message count (downlink): %s"
                      % self.packet_count)

        print(arrow_down)
        self.log.info('\n # # # # # # # # # # # # OPEN TUN # # # # # # # # # # # # ' +
                      '\n data packet EventBus -> TUN interface' +
                      '\n' + message.to_json() +
                      '\n # # # # # # # # # # # # # # # # # # # # # # # # # # # # #'
                      )

        self.tun._eventBusToTun(
            sender="Testing Tool",
            signal="tun inject",
            data=message.data
        )


class TunConnector(BaseController):
    """

    """

    NAME = "tun"

    def __init__(self, **kwargs):
        super(TunConnector, self).__init__(name=TunConnector.NAME)
        self.tun = None
        kwargs["consumer_name"] = TunConnector.NAME
        self.consumer = TunConsumer(**kwargs)
        self.consumer.log = logging.getLogger(__name__)
        self.consumer.log.setLevel(logging.DEBUG)

    def run(self):
        self.consumer.run()



# Authors:
#     Federico Sismondi <federicosismondi@gmail.com>
#     Remy Leone

from __future__ import absolute_import

from setuptools import setup, find_packages

MAJOR = 0
MINOR = 1
PATCH = 10
VERSION = '{}.{}.{}'.format(MAJOR, MINOR, PATCH)

name = 'ioppytest-agent'

CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Science/Research',
    'Intended Audience :: Developers',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Topic :: Internet',
    'Topic :: Software Development :: Testing',
    'Topic :: Scientific/Engineering',
    'Operating System :: POSIX',
    'Operating System :: Unix',
    'Operating System :: MacOS'
]

with open('version.py', 'w') as f:
    f.write('__version__ = "{version}"\n'.format(version=VERSION))

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name=name,
    author='Federico Sismondi',
    author_email='federicosismondi@gmail.com',
    maintainer='Federico Sismondi',
    maintainer_email='federicosismondi@gmail.com',
    url='https://gitlab.f-interop.eu/f-interop-contributors/agent',
    description='Component for setting up users environment for the tests',
    version=VERSION,
    license='GPLv3+',
    classifiers=CLASSIFIERS,
    packages=find_packages(exclude=['tests']),
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
        'click',
        'six',
        'kombu',
        'pika',
        'pyserial',
    ],
    entry_points={'console_scripts': ['ioppytest-agent=agent.agent_cli:main']},
)
